==> youtube_autonomous-0.0.15-py3-none-any.whl: youtube_autonomous/segments/enhancement/enhancement_element.py <==
from youtube_autonomous.segments.enums import EnhancementElementType, EnhancementElementMode
from youtube_autonomous.segments.enhancement.enhancement_element import EnhancementElement
from youtube_autonomous.segments.enhancement.objects.meme_enhancement_element import MemeEnhancementElement
from youtube_autonomous.segments.enhancement.objects.sound_enhancement_element import SoundEnhancementElement
from youtube_autonomous.segments.enhancement.objects.image_enhancement_element import ImageEnhancementElement
from youtube_autonomous.segments.enhancement.objects.sticker_enhancement_element import StickerEnhancementElement
from youtube_autonomous.segments.enhancement.objects.greenscreen_enhancement_element import GreenscreenEnhancementElement
from youtube_autonomous.segments.enhancement.objects.effect_enhancement_element import EffectEnhancementElement
from youtube_autonomous.shortcodes.consts import FILE_DURATION, SEGMENT_DURATION
from moviepy.editor import AudioFileClip, CompositeAudioClip, VideoFileClip, CompositeVideoClip, ImageClip, ColorClip, concatenate_videoclips, concatenate_audioclips
from typing import Union


class EnhancementElement:
    """
    Class that represents an element that will be used to enhance a 
    segment video by applying it. It could be an effect, a sound, a
    sticker, etc.

    This class can be built directly from a json file that is read
    as a dict, or by a previously detected shortcode that is converted
    into this.
    """
    @classmethod
    def get_class_from_type(cls, type: Union[EnhancementElementType, str]):
        """
        This method returns the EnhancementElement class that is
        equivalent of the provided 'type' EnhancementElementType.
        """
        if not type:
            raise Exception('No "type" provided.')
        
        if not isinstance(type, (EnhancementElementType, str)):
            raise Exception('The "type" parameter provided is not an EnhancementElementType nor a string.')

        if isinstance(type, str):
            if not EnhancementElementType.is_valid(type):
                raise Exception('The "type" parameter provided is not a valid EnhancementElementType value.')
            
            type = EnhancementElementType(type)

        if type == EnhancementElementType.MEME:
            return MemeEnhancementElement
        if type == EnhancementElementType.SOUND:
            return SoundEnhancementElement
        if type == EnhancementElementType.IMAGE:
            return ImageEnhancementElement
        if type == EnhancementElementType.STICKER:
            return StickerEnhancementElement
        if type == EnhancementElementType.GREEN_SCREEN:
            return GreenscreenEnhancementElement
        if type == EnhancementElementType.EFFECT:
            return EffectEnhancementElement
        
        raise Exception('The provided "type" does not have a related EnhancementElement class.')

    _type: EnhancementElementType
    """
    The enhancement element type that determines the way it will be
    built and applied.
    """
    _start: float
    """
    The time moment of the current segment in which this element is
    expected to be applied.
    """
    _duration: float
    """
    The duration of the element that the user wants to be applied.
    """
    _keywords: str = ''
    """
    The keywords that will be used to look for the resource in the 
    platform that the type of element needs, if provided.
    """
    _url: str = ''
    """
    The url from which the resource will be obtained, if set, to
    use as source of this enhancement element.
    """
    _filename: str = ''
    """
    The filename that will be used as source of this enhancement 
    element if provided.
    """
    _mode: EnhancementElementMode = EnhancementElementMode.INLINE
    """
    The mode in which the enhancement element will be applied, that could be inline
    or as overview.
    """

    def __init__(self, type: EnhancementElementType, start: float, duration: float, keywords: str, url: str, filename: str, mode: EnhancementElementMode):
        self.type = type
        self.start = start
        self.duration = duration
        self.keywords = keywords
        self.url = url
        self.filename = filename
        self.mode = mode

    @property
    def valid_modes(self):
        """
        The EnhancementElementMode enums accepted for this type of
        EnhancementElement.
        """
        return EnhancementElementMode.get_all()
    
    @valid_modes.setter
    def valid_modes(self, value):
        raise Exception('You cannot modify the "valid_modes" parameter.')

    @property
    def type(self):
        """
        The enhancement element type that determines the way it will be
        built and applied.
        """
        return self._type
    
    @type.setter
    def type(self, type: EnhancementElementType):
        if not type:
            raise Exception('No "type" provided.')
        
        if not isinstance(type, EnhancementElementType):
            raise Exception('The "type" parameter provided is not an EnhancementElementType.')
        
        self._type = type

    @property
    def start(self):
        """
        The time moment of the current segment in which this element is
        expected to be applied.
        """
        return self.start
    
    @start.setter
    def start(self, start: float):
        if not start and start != 0 and start != 0.0:
            raise Exception('No "start" provided.')
        
        if not isinstance(start, (int, float)):
            # TODO: Maybe check more number types (?)
            raise Exception('The "start" parameter provided is not a valid number.')
        
        if start < 0 or start > FILE_DURATION:
            raise Exception(f'The "start" parameter must be between 0 and {FILE_DURATION}.')
        
        self._start = start

    @property
    def duration(self):
        """
        The duration of the element that the user wants to be applied.
        """
        return self._duration
    
    @duration.setter
    def duration(self, duration: float):
        if not duration and duration != 0 and duration != 0.0:
            raise Exception('No "duration" provided.')
        
        if not isinstance(duration, (int, float)):
            # TODO: Maybe check more number types (?)
            raise Exception('The "duration" parameter provided is not a valid number.')
        
        if duration < 0 or duration > FILE_DURATION:
            raise Exception(f'The "duration" parameter must be between 0 and {FILE_DURATION}.')
        
        self._duration = duration

    @property
    def keywords(self):
        """
        The keywords that will be used to look for the resource in
        the platform that the type of element needs, if provided.
        """
        return self._keywords
    
    @keywords.setter
    def keywords(self, keywords: Union[str, None]):
        if keywords is not None and keywords != '':
            if not keywords:
                raise Exception('No "keywords" provided.')
            
            if not isinstance(keywords, str):
                raise Exception('The "keywords" parameter provided is not a string.')
        else:
            keywords = ''
        
        self._keywords = keywords

    @property
    def url(self):
        """
        The url from which the resource will be obtained, if set, to
        use as source of this enhancement element.
        """
        return self._url
    
    @url.setter
    def url(self, url: Union[str, None]):
        if url is not None and url != '':
            if not url:
                raise Exception('No "url" provided.')
            
            if not isinstance(url, str):
                raise Exception('The "url" parameter provided is not a string.')
        else:
            url = ''
        
        self._url = url

    @property
    def filename(self):
        """
        The filename that will be used as source of this enhancement 
        element if provided.
        """
        return self._filename
    
    @filename.setter
    def filename(self, filename: Union[str, None]):
        if filename is not None and filename != '':
            if not filename:
                raise Exception('No "filename" provided.')
            
            if not isinstance(filename, str):
                raise Exception('The "filename" parameter provided is not a string.')
        else:
            filename = ''
        
        self._filename = filename

    @property
    def mode(self):
        """
        The mode in which the enhancement element will be applied, that could be inline
        or as overview.
        """
        return self._mode
    
    @mode.setter
    def mode(self, mode: Union[EnhancementElementMode, str]):
        if not mode:
            raise Exception('No "mode" provided.')
        
        if not isinstance(mode, (EnhancementElementMode, str)):
            raise Exception('The "mode" parameter provided is not a EnhancementElementMode nor a string.')
        
        if isinstance(mode, str):
            if not EnhancementElementMode.is_valid(mode):
                raise Exception('The "mode" provided string is not a valid EnhancementElementMode enum value.')
            
            mode = EnhancementElementMode(mode)

        if not mode in self.valid_modes:
            raise Exception(f'The "mode" provided is not valid in this type "{self.type.value}" of EnhancementElement.')

        self._mode = mode


    def recalculate_duration(self, enhancement_clip: Union[AudioFileClip, CompositeAudioClip, VideoFileClip, CompositeVideoClip, ImageClip, ColorClip], video: Union[VideoFileClip, CompositeVideoClip, ImageClip, ColorClip]):
        """
        This method recalculates the 'self.duration' field according to
        its value, the 'self.mode' parameter value we have, and also the
        'enhancement_clip' and 'video' parameters provided.
        """
        if self.duration == FILE_DURATION:
            self.duration = enhancement_clip.duration

        if self.duration == SEGMENT_DURATION:
            self.duration = video.duration

        if self.mode in [EnhancementElementMode.INLINE, EnhancementElementMode.OVERLAY]:
            if self.duration > enhancement_clip.duration:
                self.duration = enhancement_clip.duration

        if self.mode == [EnhancementElementMode.OVERLAY, EnhancementElementMode.REPLACE]:
            end = self.start + self.duration
            if end > video.duration:
                self.duration = video.duration - self.start

    def apply(self, enhancement_clip: Union[AudioFileClip, CompositeAudioClip, VideoFileClip, CompositeVideoClip, ImageClip, ColorClip], video: Union[VideoFileClip, CompositeVideoClip, ImageClip, ColorClip]):
        """
        This method adjusts the 'enhancement_clip' duration and make the
        necessary cuts to fit in the base provided 'video' according to
        its parameters. It will consider the mode to cut the 'video' and
        insert the 'enhancement_clip' in some specific position, or will
        put it as an overlay video if requested.

        This method returns the provided 'video' with the 
        'enhancement_clip' applied on it.
        """
        if isinstance(enhancement_clip, (AudioFileClip, CompositeAudioClip)):
            audio = video.audio
            if self.mode == EnhancementElementMode.INLINE:
                if self.start == 0:
                    audio = concatenate_audioclips([
                        enhancement_clip.set_duration(self.duration),
                        audio
                    ])
                elif self.start == video.duration:
                    audio = concatenate_audioclips([
                        audio,
                        enhancement_clip.set_duration(self.duration)
                    ])
                else:
                    audio = concatenate_audioclips([
                        audio.subclip(0, self.start),
                        enhancement_clip.set_duration(self.duration),
                        audio.subclip(self.start, video.duration)
                    ])
            elif self.mode in [EnhancementElementMode.OVERLAY, EnhancementElementMode.REPLACE]:
                audio = CompositeAudioClip([
                    audio,
                    enhancement_clip.set_start(self.start).set_duration(video.duration)
                ])
            # TODO: Implement self.mode == EnhancementElementMode.REPLACE (?)

            video = video.set_audio(audio)
        elif isinstance(enhancement_clip, (VideoFileClip, CompositeVideoClip, ImageClip, ColorClip)):
            if self.mode == EnhancementElementMode.INLINE:
                if self.start == 0:
                    video = concatenate_videoclips([
                        enhancement_clip.set_duration(self.duration),
                        video
                    ])
                elif self.start == video.duration:
                    video = concatenate_videoclips([
                        video,
                        enhancement_clip.set_duration(self.duration)
                    ])
                else:
                    video = concatenate_videoclips([
                        video.subclip(0, self.start),
                        enhancement_clip.set_duration(self.duration),
                        video.subclip(self.start, video.duration)
                    ])
            elif self.mode == EnhancementElementMode.OVERLAY:
                video = CompositeVideoClip([
                    video,
                    enhancement_clip.set_start(self.start).set_duration(video.duration)
                ])
            elif self.mode == EnhancementElementMode.REPLACE:
                end = self.start + self.duration
                if self.start == 0:
                    if end == video.duration:
                        video = enhancement_clip
                    else:
                        video = concatenate_videoclips([
                            enhancement_clip,
                            video.subclip(end, video.duration)
                        ])
                elif end == video.duration:
                    video = concatenate_videoclips([
                        video.subclip(0, self.start),
                        enhancement_clip
                    ])
                else:
                    video = concatenate_videoclips([
                        video.subclip(0, self.start),
                        enhancement_clip,
                        video.subclip(end, video.duration)
                    ])

        return video

==> youtube_autonomous-0.0.21-py3-none-any.whl: youtube_autonomous/segments/enhancement/enhancement_element.py <==
from youtube_autonomous.segments.enums import EnhancementElementType, EnhancementElementMode
from youtube_autonomous.segments.enhancement.validation.segment_enhancement_validator import SegmentEnhancementValidator
from youtube_autonomous.shortcodes.consts import FILE_DURATION, SEGMENT_DURATION
from moviepy.editor import AudioFileClip, CompositeAudioClip, VideoFileClip, CompositeVideoClip, ImageClip, ColorClip, concatenate_videoclips, concatenate_audioclips
from typing import Union


class EnhancementElement:
    """
    Class that represents an element that will be used to enhance a 
    segment video by applying it. It could be an effect, a sound, a
    sticker, etc.

    This class can be built directly from a json file that is read
    as a dict, or by a previously detected shortcode that is converted
    into this.
    """
    @classmethod
    def get_class_from_type(cls, type: Union[EnhancementElementType, str]):
        """
        This method returns the EnhancementElement class that is
        equivalent of the provided 'type' EnhancementElementType.
        """
        SegmentEnhancementValidator.validate_type(type)

        if isinstance(type, str):            
            type = EnhancementElementType(type)

        from youtube_autonomous.segments.enhancement.objects.meme_enhancement_element import MemeEnhancementElement
        from youtube_autonomous.segments.enhancement.objects.sound_enhancement_element import SoundEnhancementElement
        from youtube_autonomous.segments.enhancement.objects.image_enhancement_element import ImageEnhancementElement
        from youtube_autonomous.segments.enhancement.objects.sticker_enhancement_element import StickerEnhancementElement
        from youtube_autonomous.segments.enhancement.objects.greenscreen_enhancement_element import GreenscreenEnhancementElement
        from youtube_autonomous.segments.enhancement.objects.effect_enhancement_element import EffectEnhancementElement

        if type == EnhancementElementType.MEME:
            return MemeEnhancementElement
        if type == EnhancementElementType.SOUND:
            return SoundEnhancementElement
        if type == EnhancementElementType.IMAGE:
            return ImageEnhancementElement
        if type == EnhancementElementType.STICKER:
            return StickerEnhancementElement
        if type == EnhancementElementType.GREENSCREEN:
            return GreenscreenEnhancementElement
        if type == EnhancementElementType.EFFECT:
            return EffectEnhancementElement
        
        raise Exception('The provided "type" does not have a related EnhancementElement class.')

    _type: EnhancementElementType
    """
    The enhancement element type that determines the way it will be
    built and applied.
    """
    _start: float
    """
    The time moment of the current segment in which this element is
    expected to be applied.
    """
    _duration: float
    """
    The duration of the element that the user wants to be applied.
    """
    _keywords: str = ''
    """
    The keywords that will be used to look for the resource in the 
    platform that the type of element needs, if provided.
    """
    _url: str = ''
    """
    The url from which the resource will be obtained, if set, to
    use as source of this enhancement element.
    """
    _filename: str = ''
    """
    The filename that will be used as source of this enhancement 
    element if provided.
    """
    _mode: EnhancementElementMode = EnhancementElementMode.INLINE
    """
    The mode in which the enhancement element will be applied, that could be inline
    or as overview.
    """

    def __init__(self, type: EnhancementElementType, start: float, duration: float, keywords: str, url: str, filename: str, mode: EnhancementElementMode):
        self.type = type
        self.start = start
        self.duration = duration
        self.keywords = keywords
        self.url = url
        self.filename = filename
        self.mode = mode

    @property
    def valid_modes(self):
        """
        The EnhancementElementMode enums accepted for this type of
        EnhancementElement.
        """
        return EnhancementElementMode.get_all()
    
    @valid_modes.setter
    def valid_modes(self, value):
        raise Exception('You cannot modify the "valid_modes" parameter.')

    @property
    def type(self):
        """
        The enhancement element type that determines the way it will be
        built and applied.
        """
        return self._type
    
    @type.setter
    def type(self, type: Union[EnhancementElementType, str]):
        SegmentEnhancementValidator.validate_type(type)
            
        if isinstance(type, str):
            type = EnhancementElementType(type)

        self._type = type

    @property
    def start(self):
        """
        The time moment of the current segment in which this element is
        expected to be applied.
        """
        return self.start
    
    @start.setter
    def start(self, start: float):
        SegmentEnhancementValidator.validate_start(start)
        
        self._start = start

    @property
    def duration(self):
        """
        The duration of the element that the user wants to be applied.
        """
        return self._duration
    
    @duration.setter
    def duration(self, duration: float):
        SegmentEnhancementValidator.validate_duration(duration)
        
        self._duration = duration

    @property
    def keywords(self):
        """
        The keywords that will be used to look for the resource in
        the platform that the type of element needs, if provided.
        """
        return self._keywords
    
    @keywords.setter
    def keywords(self, keywords: Union[str, None]):
        if keywords is not None and keywords != '':
            if not keywords:
                raise Exception('No "keywords" provided.')
            
            if not isinstance(keywords, str):
                raise Exception(f'The "keywords" parameter provided {str(keywords)} is not a string.')
        else:
            keywords = ''
        
        self._keywords = keywords

    @property
    def url(self):
        """
        The url from which the resource will be obtained, if set, to
        use as source of this enhancement element.
        """
        return self._url
    
    @url.setter
    def url(self, url: Union[str, None]):
        if url is not None and url != '':
            if not url:
                raise Exception('No "url" provided.')
            
            if not isinstance(url, str):
                raise Exception(f'The "url" parameter provided {str(url)} is not a string.')
        else:
            url = ''
        
        self._url = url

    @property
    def filename(self):
        """
        The filename that will be used as source of this enhancement 
        element if provided.
        """
        return self._filename
    
    @filename.setter
    def filename(self, filename: Union[str, None]):
        if filename is not None and filename != '':
            if not filename:
                raise Exception('No "filename" provided.')
            
            if not isinstance(filename, str):
                raise Exception(f'The "filename" parameter provided {str(filename)} is not a string.')
        else:
            filename = ''
        
        self._filename = filename

    @property
    def mode(self):
        """
        The mode in which the enhancement element will be applied, that could be inline
        or as overview.
        """
        return self._mode
    
    @mode.setter
    def mode(self, mode: Union[EnhancementElementMode, str]):
        SegmentEnhancementValidator.validate_mode(mode, self.valid_modes)
        
        if isinstance(mode, str):
            mode = EnhancementElementMode(mode)

        self._mode = mode


    def recalculate_duration(self, enhancement_clip: Union[AudioFileClip, CompositeAudioClip, VideoFileClip, CompositeVideoClip, ImageClip, ColorClip], video: Union[VideoFileClip, CompositeVideoClip, ImageClip, ColorClip]):
        """
        This method recalculates the 'self.duration' field according to
        its value, the 'self.mode' parameter value we have, and also the
        'enhancement_clip' and 'video' parameters provided.
        """
        if self.duration == FILE_DURATION:
            self.duration = enhancement_clip.duration

        if self.duration == SEGMENT_DURATION:
            self.duration = video.duration

        if self.mode in [EnhancementElementMode.INLINE, EnhancementElementMode.OVERLAY]:
            if self.duration > enhancement_clip.duration:
                self.duration = enhancement_clip.duration

        if self.mode == [EnhancementElementMode.OVERLAY, EnhancementElementMode.REPLACE]:
            end = self.start + self.duration
            if end > video.duration:
                self.duration = video.duration - self.start

    def apply(self, enhancement_clip: Union[AudioFileClip, CompositeAudioClip, VideoFileClip, CompositeVideoClip, ImageClip, ColorClip], video: Union[VideoFileClip, CompositeVideoClip, ImageClip, ColorClip]):
        """
        This method adjusts the 'enhancement_clip' duration and make the
        necessary cuts to fit in the base provided 'video' according to
        its parameters. It will consider the mode to cut the 'video' and
        insert the 'enhancement_clip' in some specific position, or will
        put it as an overlay video if requested.

        This method returns the provided 'video' with the 
        'enhancement_clip' applied on it.
        """
        if isinstance(enhancement_clip, (AudioFileClip, CompositeAudioClip)):
            audio = video.audio
            if self.mode == EnhancementElementMode.INLINE:
                if self.start == 0:
                    audio = concatenate_audioclips([
                        enhancement_clip.set_duration(self.duration),
                        audio
                    ])
                elif self.start == video.duration:
                    audio = concatenate_audioclips([
                        audio,
                        enhancement_clip.set_duration(self.duration)
                    ])
                else:
                    audio = concatenate_audioclips([
                        audio.subclip(0, self.start),
                        enhancement_clip.set_duration(self.duration),
                        audio.subclip(self.start, video.duration)
                    ])
            elif self.mode in [EnhancementElementMode.OVERLAY, EnhancementElementMode.REPLACE]:
                audio = CompositeAudioClip([
                    audio,
                    enhancement_clip.set_start(self.start).set_duration(video.duration)
                ])
            # TODO: Implement self.mode == EnhancementElementMode.REPLACE (?)

            video = video.set_audio(audio)
        elif isinstance(enhancement_clip, (VideoFileClip, CompositeVideoClip, ImageClip, ColorClip)):
            if self.mode == EnhancementElementMode.INLINE:
                if self.start == 0:
                    video = concatenate_videoclips([
                        enhancement_clip.set_duration(self.duration),
                        video
                    ])
                elif self.start == video.duration:
                    video = concatenate_videoclips([
                        video,
                        enhancement_clip.set_duration(self.duration)
                    ])
                else:
                    video = concatenate_videoclips([
                        video.subclip(0, self.start),
                        enhancement_clip.set_duration(self.duration),
                        video.subclip(self.start, video.duration)
                    ])
            elif self.mode == EnhancementElementMode.OVERLAY:
                video = CompositeVideoClip([
                    video,
                    enhancement_clip.set_start(self.start).set_duration(video.duration)
                ])
            elif self.mode == EnhancementElementMode.REPLACE:
                end = self.start + self.duration
                if self.start == 0:
                    if end == video.duration:
                        video = enhancement_clip
                    else:
                        video = concatenate_videoclips([
                            enhancement_clip,
                            video.subclip(end, video.duration)
                        ])
                elif end == video.duration:
                    video = concatenate_videoclips([
                        video.subclip(0, self.start),
                        enhancement_clip
                    ])
                else:
                    video = concatenate_videoclips([
                        video.subclip(0, self.start),
                        enhancement_clip,
                        video.subclip(end, video.duration)
                    ])

        return video
